package com.huiju.estateagents.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.common.CommConstant;
import com.huiju.estateagents.entity.TaPersonVisitRecord;
import com.huiju.estateagents.exception.EstaException;
import com.huiju.estateagents.mapper.TaBuildingMapper;
import com.huiju.estateagents.mapper.TaPersonMapper;
import com.huiju.estateagents.mapper.TaPersonVisitRecordMapper;
import com.huiju.estateagents.po.PersonPO;
import com.huiju.estateagents.service.IStatisticalService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 数据统计
 */
@Service
@Slf4j
public class StatisticalServiceImpl implements IStatisticalService {

    @Autowired
    private TaPersonMapper taPersonMapper;

    @Autowired
    private TaBuildingMapper taBuildingMapper;


    @Autowired
    private TaPersonVisitRecordMapper taPersonVisitRecordMapper;

    @Override
    public ResponseBean indexStatistical() {

        ResponseBean responseBean = new ResponseBean();
        Map<String,Object> map = new HashMap<>();

        // 总用户数
        Integer selectUserCount = taPersonMapper.selectUserCount(CommConstant.PERSON_REALTY_CONSULTANT);

        // 注册数
        Integer selectRegisteredCount = taPersonMapper.selectRegisteredCount(CommConstant.PERSON_REALTY_CONSULTANT);

        // 最近七天
        Integer selectRecentlyCount = taPersonMapper.selectRecentlyCount(CommConstant.PERSON_REALTY_CONSULTANT, null, null);

        // 用户行为
        List<PersonPO> selectUserBehavior = taPersonMapper.selectUserBehavior(CommConstant.PERSON_REALTY_CONSULTANT, null, null);

        // 用户活跃数
        List<Map<String, Object>> selectActiveUserCount = taPersonMapper.selectActiveUserCount(CommConstant.PERSON_REALTY_CONSULTANT, CommConstant.DAY, null, null);

        // 新增用户数
        List<Map<String, Object>> selectNewsUserCount = taPersonMapper.selectNewsUserCount(CommConstant.PERSON_REALTY_CONSULTANT, null, null);

        // 性别比例
        List<Map<String, Object>> selectSexUser = taPersonMapper.selectSexUser(CommConstant.PERSON_REALTY_CONSULTANT);

        // 城市比例
        List<Map<String, Object>> selectCityUser = taPersonMapper.selectCityUser(CommConstant.PERSON_REALTY_CONSULTANT);

        // 转化率, 当前 默认项目收藏
        Map<String, Object> selectBuildingStatistical = taBuildingMapper.selectBuildingStatistical("save");

        map.put("selectUserCount", selectUserCount);
        map.put("selectRegisteredCount", selectRegisteredCount);
        map.put("selectRecentlyCount", selectRecentlyCount);
        map.put("selectUserBehavior", selectUserBehavior);
        map.put("selectActiveUserCount", selectActiveUserCount);
        map.put("selectNewsUserCount", selectNewsUserCount);
        map.put("selectSexUser", selectSexUser);
        map.put("selectCityUser", selectCityUser);
        map.put("selectBuildingStatistical", selectBuildingStatistical);

        responseBean.addSuccess(map);

        return responseBean;
    }

    @Override
    public ResponseBean selectActiveUserCount(String dateType) {
        ResponseBean responseBean = new ResponseBean();

        String type = null;
        if (CommConstant.DAY.equals(dateType)) {
            type = CommConstant.DAY;
        } else if (CommConstant.WEEK.equals(dateType)) {
            type = CommConstant.WEEK;
        } else if (CommConstant.MONTH.equals(dateType)) {
            type = CommConstant.MONTH;
        } else {
            responseBean.addError("无此查询类型！");
        }

        // 用户活跃数
        List<Map<String, Object>> selectActiveUserCount = taPersonMapper.selectActiveUserCount(CommConstant.PERSON_REALTY_CONSULTANT, type, null, null);
        Map<String, Object> map = new HashMap<>();
        map.put("selectActiveUserCount", selectActiveUserCount);
        responseBean.addSuccess(map);
        return responseBean;
    }

    @Override
    public ResponseBean selectNewsUserCount(Date startDate, Date endDate) {
        ResponseBean responseBean = new ResponseBean();

        // 新增用户数
        List<Map<String, Object>> selectNewsUserCount = taPersonMapper.selectNewsUserCount(CommConstant.PERSON_REALTY_CONSULTANT, startDate, endDate);

        Map<String, Object> map = new HashMap<>();
        map.put("selectNewsUserCount", selectNewsUserCount);
        responseBean.addSuccess(map);
        return responseBean;
    }

    @Override
    public ResponseBean selectUserBehavior(Date startDate, Date endDate) {
        ResponseBean responseBean = new ResponseBean();

        // 用户行为
        List<PersonPO> selectUserBehavior = taPersonMapper.selectUserBehavior(CommConstant.PERSON_REALTY_CONSULTANT, startDate, endDate);

        Map<String, Object> map = new HashMap<>();
        map.put("selectUserBehavior", selectUserBehavior);
        responseBean.addSuccess(map);
        return responseBean;
    }

    @Override
    public ResponseBean selectUserBehaviorAll(Integer pageNum, Integer pageSize) {
        ResponseBean responseBean = new ResponseBean();
        IPage<TaPersonVisitRecord> page = new Page<>(pageNum, pageSize);
        IPage<TaPersonVisitRecord> visitRecordIPage = taPersonVisitRecordMapper.selectAll(page);
        responseBean.addSuccess(visitRecordIPage);
        return responseBean;
    }

    @Override
    public ResponseBean selectEventAll(String event, Integer personId) {
        ResponseBean responseBean = new ResponseBean();

        List<TaPersonVisitRecord> taPersonVisitRecords = taPersonVisitRecordMapper.selectEventAll(event, personId);
        responseBean.addSuccess(taPersonVisitRecords);
        return responseBean;
    }
}
